@@ -166,6 +166,29 @@ struct DeviceMergeSort
                                                                              compare_op,
                                                                              stream));
     }
+
+    template<typename KeyInputIteratorT,
+             typename KeyIteratorT,
+             typename OffsetT,
+             typename CompareOpT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t StableSortKeysCopy(void*        d_temp_storage,
+                                                                 std::size_t& temp_storage_bytes,
+                                                                 KeyInputIteratorT d_input_keys,
+                                                                 KeyIteratorT      d_output_keys,
+                                                                 OffsetT           num_items,
+                                                                 CompareOpT        compare_op,
+                                                                 hipStream_t       stream = 0,
+                                                                 bool debug_synchronous   = false)
+    {
+        (void)debug_synchronous;
+        return hipCUDAErrorTohipError(::cub::DeviceMergeSort::StableSortKeysCopy(d_temp_storage,
+                                                                                 temp_storage_bytes,
+                                                                                 d_input_keys,
+                                                                                 d_output_keys,
+                                                                                 num_items,
+                                                                                 compare_op,
+                                                                                 stream));
+    }
 };
 
 END_HIPCUB_NAMESPACE
