@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2017-2020, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2017-2023, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -162,64 +162,65 @@ inline hip_bfloat16 get_max_special_value<hip_bfloat16>()
 class DeviceReduce
 {
 public:
-    template <
-        typename InputIteratorT,
-        typename OutputIteratorT,
-        typename ReduceOpT,
-        typename T
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t Reduce(void *d_temp_storage,
-                      size_t &temp_storage_bytes,
-                      InputIteratorT d_in,
-                      OutputIteratorT d_out,
-                      int num_items,
-                      ReduceOpT reduction_op,
-                      T init,
-                      hipStream_t stream = 0,
-                      bool debug_synchronous = false)
+    template<typename InputIteratorT,
+             typename OutputIteratorT,
+             typename ReduceOpT,
+             typename T,
+             typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t Reduce(void*           d_temp_storage,
+                                                     size_t&         temp_storage_bytes,
+                                                     InputIteratorT  d_in,
+                                                     OutputIteratorT d_out,
+                                                     NumItemsT       num_items,
+                                                     ReduceOpT       reduction_op,
+                                                     T               init,
+                                                     hipStream_t     stream            = 0,
+                                                     bool            debug_synchronous = false)
     {
         return ::rocprim::reduce(
-            d_temp_storage, temp_storage_bytes,
-            d_in, d_out, init, num_items,
-            ::hipcub::detail::convert_result_type<InputIteratorT, OutputIteratorT>(reduction_op),
-            stream, debug_synchronous
-        );
+            d_temp_storage,
+            temp_storage_bytes,
+            d_in,
+            d_out,
+            init,
+            num_items,
+            ::hipcub::detail::convert_binary_result_type<T, InputIteratorT, OutputIteratorT>(
+                reduction_op),
+            stream,
+            debug_synchronous);
     }
 
-    template <
-        typename InputIteratorT,
-        typename OutputIteratorT
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t Sum(void *d_temp_storage,
-                   size_t &temp_storage_bytes,
-                   InputIteratorT d_in,
-                   OutputIteratorT d_out,
-                   int num_items,
-                   hipStream_t stream = 0,
-                   bool debug_synchronous = false)
+    template<typename InputIteratorT, typename OutputIteratorT, typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t Sum(void*           d_temp_storage,
+                                                  size_t&         temp_storage_bytes,
+                                                  InputIteratorT  d_in,
+                                                  OutputIteratorT d_out,
+                                                  NumItemsT       num_items,
+                                                  hipStream_t     stream            = 0,
+                                                  bool            debug_synchronous = false)
     {
-        using T = typename std::iterator_traits<InputIteratorT>::value_type;
-        return Reduce(
-            d_temp_storage, temp_storage_bytes,
-            d_in, d_out, num_items, ::hipcub::Sum(), T(0),
-            stream, debug_synchronous
-        );
+        using InputT  = typename std::iterator_traits<InputIteratorT>::value_type;
+        using OutputT = typename std::iterator_traits<OutputIteratorT>::value_type;
+        using InitT   = hipcub::detail::non_void_value_t<OutputT, InputT>;
+        return Reduce(d_temp_storage,
+                      temp_storage_bytes,
+                      d_in,
+                      d_out,
+                      num_items,
+                      ::hipcub::Sum(),
+                      InitT(0),
+                      stream,
+                      debug_synchronous);
     }
 
-    template <
-        typename InputIteratorT,
-        typename OutputIteratorT
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t Min(void *d_temp_storage,
-                   size_t &temp_storage_bytes,
-                   InputIteratorT d_in,
-                   OutputIteratorT d_out,
-                   int num_items,
-                   hipStream_t stream = 0,
-                   bool debug_synchronous = false)
+    template<typename InputIteratorT, typename OutputIteratorT, typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t Min(void*           d_temp_storage,
+                                                  size_t&         temp_storage_bytes,
+                                                  InputIteratorT  d_in,
+                                                  OutputIteratorT d_out,
+                                                  NumItemsT       num_items,
+                                                  hipStream_t     stream            = 0,
+                                                  bool            debug_synchronous = false)
     {
         using T = typename std::iterator_traits<InputIteratorT>::value_type;
         return Reduce(
@@ -229,28 +230,19 @@ public:
         );
     }
 
-    template <
-        typename InputIteratorT,
-        typename OutputIteratorT
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t ArgMin(void *d_temp_storage,
-                      size_t &temp_storage_bytes,
-                      InputIteratorT d_in,
-                      OutputIteratorT d_out,
-                      int num_items,
-                      hipStream_t stream = 0,
-                      bool debug_synchronous = false)
+    template<typename InputIteratorT, typename OutputIteratorT, typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t ArgMin(void*           d_temp_storage,
+                                                     size_t&         temp_storage_bytes,
+                                                     InputIteratorT  d_in,
+                                                     OutputIteratorT d_out,
+                                                     NumItemsT       num_items,
+                                                     hipStream_t     stream            = 0,
+                                                     bool            debug_synchronous = false)
     {
-        using OffsetT = int;
+        using OffsetT      = NumItemsT;
         using T = typename std::iterator_traits<InputIteratorT>::value_type;
         using O = typename std::iterator_traits<OutputIteratorT>::value_type;
-        using OutputTupleT =
-            typename std::conditional<
-                std::is_same<O, void>::value,
-                KeyValuePair<OffsetT, T>,
-                O
-            >::type;
+        using OutputTupleT = hipcub::detail::non_void_value_t<O, KeyValuePair<OffsetT, T>>;
 
         using OutputValueT = typename OutputTupleT::Value;
         using IteratorT = ArgIndexInputIterator<InputIteratorT, OffsetT, OutputValueT>;
@@ -269,18 +261,14 @@ public:
         );
     }
 
-    template <
-        typename InputIteratorT,
-        typename OutputIteratorT
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t Max(void *d_temp_storage,
-                   size_t &temp_storage_bytes,
-                   InputIteratorT d_in,
-                   OutputIteratorT d_out,
-                   int num_items,
-                   hipStream_t stream = 0,
-                   bool debug_synchronous = false)
+    template<typename InputIteratorT, typename OutputIteratorT, typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t Max(void*           d_temp_storage,
+                                                  size_t&         temp_storage_bytes,
+                                                  InputIteratorT  d_in,
+                                                  OutputIteratorT d_out,
+                                                  NumItemsT       num_items,
+                                                  hipStream_t     stream            = 0,
+                                                  bool            debug_synchronous = false)
     {
         using T = typename std::iterator_traits<InputIteratorT>::value_type;
         return Reduce(
@@ -290,28 +278,19 @@ public:
         );
     }
 
-    template <
-        typename InputIteratorT,
-        typename OutputIteratorT
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t ArgMax(void *d_temp_storage,
-                      size_t &temp_storage_bytes,
-                      InputIteratorT d_in,
-                      OutputIteratorT d_out,
-                      int num_items,
-                      hipStream_t stream = 0,
-                      bool debug_synchronous = false)
+    template<typename InputIteratorT, typename OutputIteratorT, typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t ArgMax(void*           d_temp_storage,
+                                                     size_t&         temp_storage_bytes,
+                                                     InputIteratorT  d_in,
+                                                     OutputIteratorT d_out,
+                                                     NumItemsT       num_items,
+                                                     hipStream_t     stream            = 0,
+                                                     bool            debug_synchronous = false)
     {
-        using OffsetT = int;
+        using OffsetT      = NumItemsT;
         using T = typename std::iterator_traits<InputIteratorT>::value_type;
         using O = typename std::iterator_traits<OutputIteratorT>::value_type;
-        using OutputTupleT =
-            typename std::conditional<
-                std::is_same<O, void>::value,
-                KeyValuePair<OffsetT, T>,
-                O
-            >::type;
+        using OutputTupleT = hipcub::detail::non_void_value_t<O, KeyValuePair<OffsetT, T>>;
 
         using OutputValueT = typename OutputTupleT::Value;
         using IteratorT = ArgIndexInputIterator<InputIteratorT, OffsetT, OutputValueT>;
@@ -330,37 +309,40 @@ public:
         );
     }
 
-    template<
-        typename KeysInputIteratorT,
-        typename UniqueOutputIteratorT,
-        typename ValuesInputIteratorT,
-        typename AggregatesOutputIteratorT,
-        typename NumRunsOutputIteratorT,
-        typename ReductionOpT
-    >
-    HIPCUB_RUNTIME_FUNCTION static
-    hipError_t ReduceByKey(void * d_temp_storage,
-                           size_t& temp_storage_bytes,
-                           KeysInputIteratorT d_keys_in,
-                           UniqueOutputIteratorT d_unique_out,
-                           ValuesInputIteratorT d_values_in,
-                           AggregatesOutputIteratorT d_aggregates_out,
-                           NumRunsOutputIteratorT d_num_runs_out,
-                           ReductionOpT reduction_op,
-                           int num_items,
-                           hipStream_t stream = 0,
-                           bool debug_synchronous = false)
+    template<typename KeysInputIteratorT,
+             typename UniqueOutputIteratorT,
+             typename ValuesInputIteratorT,
+             typename AggregatesOutputIteratorT,
+             typename NumRunsOutputIteratorT,
+             typename ReductionOpT,
+             typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION static hipError_t
+        ReduceByKey(void*                     d_temp_storage,
+                    size_t&                   temp_storage_bytes,
+                    KeysInputIteratorT        d_keys_in,
+                    UniqueOutputIteratorT     d_unique_out,
+                    ValuesInputIteratorT      d_values_in,
+                    AggregatesOutputIteratorT d_aggregates_out,
+                    NumRunsOutputIteratorT    d_num_runs_out,
+                    ReductionOpT              reduction_op,
+                    NumItemsT                 num_items,
+                    hipStream_t               stream            = 0,
+                    bool                      debug_synchronous = false)
     {
         using key_compare_op =
             ::rocprim::equal_to<typename std::iterator_traits<KeysInputIteratorT>::value_type>;
-        return ::rocprim::reduce_by_key(
-            d_temp_storage, temp_storage_bytes,
-            d_keys_in, d_values_in, num_items,
-            d_unique_out, d_aggregates_out, d_num_runs_out,
-            ::hipcub::detail::convert_result_type<ValuesInputIteratorT, AggregatesOutputIteratorT>(reduction_op),
-            key_compare_op(),
-            stream, debug_synchronous
-        );
+        return ::rocprim::reduce_by_key(d_temp_storage,
+                                        temp_storage_bytes,
+                                        d_keys_in,
+                                        d_values_in,
+                                        num_items,
+                                        d_unique_out,
+                                        d_aggregates_out,
+                                        d_num_runs_out,
+                                        reduction_op,
+                                        key_compare_op(),
+                                        stream,
+                                        debug_synchronous);
     }
 };
 
