@@ -85,34 +85,34 @@ public:
     }
 
     HIPCUB_HOST_DEVICE inline
-    reference operator[](difference_type n) const
+    reference operator[](const difference_type& n) const
     {
         return *(ptr_ + n);
     }
 
     HIPCUB_HOST_DEVICE inline
-    identity_iterator operator+(difference_type distance) const
+    identity_iterator operator+(const difference_type& distance) const
     {
         auto i = ptr_ + distance;
         return identity_iterator(i);
     }
 
     HIPCUB_HOST_DEVICE inline
-    identity_iterator& operator+=(difference_type distance)
+    identity_iterator& operator+=(const difference_type& distance)
     {
         ptr_ += distance;
         return *this;
     }
 
     HIPCUB_HOST_DEVICE inline
-    identity_iterator operator-(difference_type distance) const
+    identity_iterator operator-(const difference_type& distance) const
     {
         auto i = ptr_ - distance;
         return identity_iterator(i);
     }
 
     HIPCUB_HOST_DEVICE inline
-    identity_iterator& operator-=(difference_type distance)
+    identity_iterator& operator-=(const difference_type& distance)
     {
         ptr_ -= distance;
         return *this;
