@@ -93,7 +93,7 @@ struct TransformOp
 struct SelectOp
 {
     template <typename T>
-    __host__ __device__ __forceinline__ bool operator()(T input)
+    __host__ __device__ __forceinline__ bool operator()(T input) const
     {
         (void) input;
         return true;
@@ -346,8 +346,7 @@ TYPED_TEST(HipcubIteratorTests, TestTexObj)
     std::string deviceName = std::string(props.gcnArchName);
     if (deviceName.rfind("gfx94", 0) == 0) {
         // This is a gfx94x device, so skip this test
-        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
-        return;
+        GTEST_SKIP() << "Test not run on gfx94x as texture cache API is not supported";
     }
 
     HIP_CHECK(hipSetDevice(device_id));
@@ -414,8 +413,7 @@ TYPED_TEST(HipcubIteratorTests, TestTexRef)
     std::string deviceName = std::string(props.gcnArchName);
     if (deviceName.rfind("gfx94", 0) == 0) {
         // This is a gfx94x device, so skip this test
-        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
-        return;
+        GTEST_SKIP() << "Test not run on gfx94x as texture cache API is not supported";
     }
 
     HIP_CHECK(hipSetDevice(device_id));
@@ -486,8 +484,7 @@ TYPED_TEST(HipcubIteratorTests, TestTexTransform)
     std::string deviceName = std::string(props.gcnArchName);
     if (deviceName.rfind("gfx94", 0) == 0) {
         // This is a gfx94x device, so skip this test
-        SCOPED_TRACE(testing::Message() << "Skipping texture cache text for " << deviceName);
-        return;
+        GTEST_SKIP() << "Test not run on gfx94x as texture cache API is not supported";
     }
 
     HIP_CHECK(hipSetDevice(device_id));
